#!/usr/bin/env python
##------------------------------------------------------------------------------
##
## export_saa_dark.py
##
## We want to "export" (copy?) SAA Dark files to the "cache"
##
## History:
## mm/dd/yy PR    Who        What          
## -------- ----- ---------- --------------------------------------------------
## 12/29/08 60896 Sherbert   remove from ac_xxx.csh
## 08/12/09 63318 Sherbert   easier debug if don't import *
##
##------------------------------------------------------------------------------

import os
import sys
import opusutil
import ac_pkg

## Sometimes maybe you want the message to look
## like it comes from the calling program
moduleName = os.path.basename(sys.argv[0])
## but errors should specify this script
thisModule = 'export_saa_dark'

##------------------------------------------------------------------------------
##
## Method:  usage()
##
## Description: 
##          Print usage message for export_saa_dark(.py) 
##
## Returns: nothing
##
##------------------------------------------------------------------------------
def usage():
    usageMsg = """
            export_saa_dark.py

            If no command-line arguments are provided, environment variables 
            must be defined for the 3 arguments.  Otherwise, the arguments are 
            expected to be in this order:

            <SAADARK_CACHE> <OSF_DATASET> <OUTPATH>

            <SAADARK_CACHE> is the dir to contain SAA dark files
            <OSF_DATASET> is expected to exist in <OUTPATH>.  
            <KW_DB> ingest_files_mapping table on <KW_SERVER>.

            This script will copy some SAA-related NIC files.  Currently
            the ASN with the same root as the saa file and it's two raw
            members are being copied.  Have an email in to Vicki Laidler 
            to see if this is correct.  I would think we want the saa 
            file?  However, that is not what is on the operational system...

    """
    opusutil.PrintMsg('F', usageMsg, thisModule)
    return


##------------------------------------------------------------------------------
##
## Method: copyFile
##
## Description: copy sourceFile to destination
##         Uses shutil.copy which creates or overwrites file at destination,
##         copies permission bits, but does NOT copy last-access or
##         modification times (that requires copy2).
##
## Input:
##      sourceFile  - name of a file
##      destination - name of desination dir or file
##
## Returns:
##       ALL_IS_WELL if copy is successful
##       QUIT_THIS_DATASET if an IOError is detected
##
##------------------------------------------------------------------------------
def copyFile(sourceFile, destination):
    import shutil

    baseFile = os.path.basename(sourceFile)
    try:
        msg = "cp " + baseFile + " to SAADARK_CACHE "
        opusutil.PrintMsg('V',msg)
        shutil.copy( baseFile, destination )
    except IOError, extraInfo:
        strExtraInfo = str(extraInfo)+" shutil.copy FAILED"
        opusutil.PrintMsg ('F', strExtraInfo, thisModule )
        return ac_pkg.QUIT_THIS_DATASET

    return ac_pkg.ALL_IS_WELL

##------------------------------------------------------------------------------
##
## Method: doIt
##
## Input:
##     a hash keyed on SAADARK_CACHE, OSF_DATASET, and OUTPATH
##
## Description: 
##     worker bee, heart of program, callable from another python script
##
##------------------------------------------------------------------------------
def doIt(inputVars):

    SAADARK_CACHE = inputVars["SAADARK_CACHE"]
    rootname  = inputVars["OSF_DATASET"]
    ## Maybe I do not need this one?
    caldir    = inputVars["OUTPATH"]

    ## See if there is anything to do
    saaDarkFile = rootname + "_saa.fits"
    if not os.path.exists(saaDarkFile):
        ## There is nothing to do
        msg = saaDarkFile + " does NOT exist.  Nothing to do. "
        opusutil.PrintMsg('I', msg, moduleName)
        return

    ## Seems a little late but now we are going to do something
    msg = '-- Starting  export_saa_dark for '+rootname
    opusutil.PrintMsg('I', msg, moduleName )

    cache_export_error = 0

    ## SAADARK_CACHE must exist
    if not os.path.exists(SAADARK_CACHE):
        msg = "SAADARK_CACHE directory does not exist: " + SAADARK_CACHE
        opusutil.PrintMsg('F', msg, thisModule )
        return ac_pkg.GO_ABSENT

    saaASNfile = rootname + "_asn.fits"
    if not os.path.exists(saaASNfile):
        msg = "SAA DARK ASN file not found: " + saaASNfile 
        opusutil.PrintMsg('E', msg, thisModule )
        cache_export_error = cache_export_error + 1
        return cache_export_error 

    try:
        members = ac_pkg.get_lc_ASNmembers( rootname )
    except RuntimeError, errMsg:
        opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.get_lc_ASNmembers')
        return ac_pkg.QUIT_THIS_DATASET
    for member in members:
        rawMember = member + "_raw.fits"
        if os.path.exists ( rawMember ):
            ## Copy each member to the SAADARK_CACHE dir
            stat = copyFile(rawMember, SAADARK_CACHE)
            if stat > 0:
                msg  = "SAADARK MEM " + rawMember  
                msg += " FAILED to copy to disk cache " 
                msg += SAADARK_CACHE 
                opusutil.PrintMsg('E', msg, thisModule )
                cache_export_error = cache_export_error + 1
        else:
            msg = "MEMBER does NOT exist: " + rawMember
            opusutil.PrintMsg('E', msg)
            cache_export_error = cache_export_error + 1

    ## Now that the members are copied, also copy the ASN file
    ## Since filling of the cache is not critical, we will attempt to
    ## copy the ASN file even if there was a problem with the members
    stat = copyFile(saaASNfile, SAADARK_CACHE)
    if stat > 0:
        msg  = "SAADARK ASN " + saaASNfile + " FAILED to copy to disk cache " 
        msg += SAADARK_CACHE 
        opusutil.PrintMsg('E', msg, thisModule )
        cache_export_error = cache_export_error + 1

## Can this happen if stat == 0?
    if cache_export_error > 0:
        return cache_export_error  

    msg  = "SAADARK (" + rootname + ") and members " 
    msg += "have been COPIED to disk cache " 
    opusutil.PrintMsg('I', msg, moduleName)
    debug = "cache is " + SAADARK_CACHE 
    opusutil.PrintMsg('D', debug, moduleName)
    msg = '-- Completed export_saa_dark for '+rootname
    opusutil.PrintMsg('I', msg, moduleName )
    return

##------------------------------------------------------------------------------
##
## Main
##
##------------------------------------------------------------------------------
if __name__ == "__main__":
    ac_pkg.banner()
    ## Want this script to be runnable from command line or from pipeline
    ## or callable from another python script
    envVars = {}
    needThese = ["SAADARK_CACHE", "OSF_DATASET", "OUTPATH"]
    if (len(sys.argv)) == 1:
        opusutil.PrintMsg('D', 'Processing env vars', moduleName )
        try:    
            envVars = ac_pkg.envSetUp('env', needThese)
        except RuntimeError, errMsg: 
            opusutil.PrintMsg('F', str(errMsg), thisModule )
            usage() 
            sys.exit(ac_pkg.GO_ABSENT)
    else:   
        opusutil.PrintMsg('D', 'Processing cmd line vars', moduleName )
        envVars = ac_pkg.envSetUp('cmdLine', needThese)

    ## Verify and cd to caldir
    caldir = envVars["OUTPATH"]
    try:
        ac_pkg.verify_n_cd_to_dir(caldir)
    except RuntimeError, errMsg:
        opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.verify_n_cd_to_dir')
        sys.exit(ac_pkg.GO_ABSENT)

    stat = doIt(envVars)
    if stat == ac_pkg.GO_ABSENT:
        ac_pkg.banner()
        sys.exit(ac_pkg.GO_ABSENT)
    elif stat == ac_pkg.QUIT_THIS_DATASET:
        ac_pkg.banner()
        sys.exit(ac_pkg.QUIT_THIS_DATASET)
    
    sys.exit(ac_pkg.ALL_IS_WELL)

